const fs = require('fs');
const yml = require('js-yaml');
const merge = require('lodash.merge');
const defaultConfig = require('./extract-config.json');
const beautify = require('js-beautify').js;

async function extract(obj = {type: []}) {
    let {
        url = obj.url,
        id = obj.id,
        username = obj.username,
        password = obj.password,
        folder = obj.folder,
        withJson = obj.withJson,
        type = obj.type
    } = require('minimist')(process.argv.slice(2));

    if (!id) { throw new Error('--id expected')}
    
    if (!url) {
        url = defaultConfig.url;
        console.warn('No --url provided, default to: ', defaultConfig.url);
    }
    if (!username) {
        username = defaultConfig.username;
        console.warn('No --username provided, default to: ', defaultConfig.username);
    }
    if (!password) {
        password = defaultConfig.password;
        console.warn('No --password provided, default to default config');
    }

    const common = require('./extract-common.js');
    common.setUrl(url);
    await common.logout();
    await common.login(username, password)

    let finalArray = []
    if (id === '*' && type.length) {
        if (typeof type === 'string') {
            type = type.split(',').map(typeItem => {
                try {
                    const lookupTypeInt = parseInt(typeItem)
                    if (!isNaN(lookupTypeInt)) {
                        typeItem = `lookups:${typeItem}`
                    }
                }
                catch (e) {}

                return typeItem;
            });
        }

        for(const eachType of type) {
            if (eachType === 'items') {
                const itemsArray = await common.Items.extract(folder)
                finalArray = finalArray.concat(itemsArray);
                console.log('finalArray - current length:', finalArray.length);
            }
            else if (eachType.indexOf('lookups:') > -1) {
                const [lookupFlag, lookupType] = eachType.split(':');
                try {
                    console.log('Extracting L' + lookupType);
                    const lookupArray = await common.Lookups.extract(lookupType, folder);
                    finalArray = finalArray.concat(lookupArray);
                }
                catch (e) { console.error(e); }
                console.log('finalArray - current length:', finalArray.length);
            }
        }
    }
    else {
        const ids = id.split(',').map(id => id.trim());
        for (const id of ids) {
            let isAdded = false
            try {
                const lookup = await common.getLookup(id)
                let lookupArray = await common.Lookup.getDependencies(lookup, folder)
                finalArray = finalArray.concat(lookupArray);
                console.log('lookupArray - length:', lookupArray.length);
                isAdded = true
            }
            catch (error) {
                isAdded = false
            }
    
            if (!isAdded) {
                try {
                    const item = await (new common.Item(id, folder)).extract();
                    finalArray.push(item)
                    console.log('item pushed - arrayLength:', finalArray.length);
                    isAdded = true
                }
                catch (error) {
                    console.log('extracting Item, error:', error);
                    isAdded = false
                }
            }
        }
    }

    console.log('finalArray - length:', finalArray.length);
    const appMaps = {};
    /**
     * `_APP` does not represent `appcode`, it is a flag to indicate same type of `object`
     */
    const toSave = finalArray.map((finalArrayItem, i) => {
        if (finalArrayItem._APP) {
            if (
                typeof appMaps[finalArrayItem._APP] === 'number' &&
                appMaps[finalArrayItem._APP] > -1 && 
                finalArray[appMaps[finalArrayItem._APP]] &&
                finalArray[appMaps[finalArrayItem._APP]].key === finalArrayItem.key
            ) {
                merge(finalArray[appMaps[finalArrayItem._APP]], finalArrayItem)
                return undefined
            }
            else {
                appMaps[finalArrayItem._APP] = i
            }
        }

        return finalArrayItem
    }, {}).filter(item => item);

    toSave.map(toYml => {
        let forJsoning = JSON.parse(JSON.stringify(toYml));
        delete forJsoning._APP;
        forJsoning = JSON.stringify(forJsoning).replace(/\\t/gm, '    ').trim()

        return {
            key: toYml._APP ? toYml.key : undefined,
            id: toYml.id,
            name: toYml.name,
            APP: toYml.APP,
            _APP: toYml._APP,
            type: toYml.APP || (typeof toYml.type === 'number' ? toYml.type.toString() : toYml.type) || toYml._APP,
            json: yml.load(forJsoning)
        };
    }).forEach(objWithYml => {
        let fileName = objWithYml.APP ? `${objWithYml.APP}__${objWithYml.id}` : 
                         objWithYml._APP ? `${objWithYml.key}` :
                         `${(objWithYml.name || 'ERR').replace(/\W/g, ' ')} (${objWithYml.id})`;
        fileName = fileName || 'ERR'
        delete objWithYml._APP;
        
        try { fs.mkdirSync(`./${objWithYml.type}`); } catch (e) { }
        try {
            fs.writeFileSync(`./${objWithYml.type}/${fileName}.yaml`,  yml.dump(objWithYml.json));
            console.log(`./${objWithYml.type}/${fileName}.yaml` + ' created.');
        }
        catch (e) {}
        try {
            if (withJson) {
                fs.writeFileSync(`./${objWithYml.type}/${fileName}.json`,  beautify(JSON.stringify(objWithYml.json)));
                console.log(`./${objWithYml.type}/${fileName}.json` + ' created.');
            }
        }
        catch (e) {}
    })

    await common.logout();
    return
}

module.exports = {
    extract
}