const app = require('../package.json')
const Configstore = require('configstore')
const config = new Configstore(app.name)
const fetch = require('node-fetch')
const fs = require('fs')

const SLUGIFY_STRIP_RE = /[^a-zA-Z0-9\-\._]/g
const SLUGIFY_HYPHENATE_RE = /[-\s]+/g

exports.slugify = (s) => {
  s = s.replace(SLUGIFY_STRIP_RE, '').trim().toLowerCase()
  s = s.replace(SLUGIFY_HYPHENATE_RE, '-')
  return s
}

exports.config = () => {
  return config
}

exports.login = async (url, token) => {
  // login - if successful then save to config
  url = url || config.get('url')
  url = url.endsWith('/') ? url : `${url}/`
  let ok = false
  try {
    const res = await fetch(`${url}api/data/app-lookups`, {
      headers: { 'Authorization': `Bearer ${token}` }
    })
    , data = await res.json()
    ok = data.ok
    if (!data.ok) console.error(data.errorCode, data.errorMsg)
  }
  catch (err) {
    console.log(err.message)
  }
  if (ok) config.set({ url, token })
  return ok
}

exports.logout = () => {
  config.set({ url: '', token: '' })
}

exports.upload = async (file, { install = false, release = false }) => {
  // POST api/sys/pack
  // ?install=1
  // ?release=1
  const FormData = require('form-data')
  const form = new FormData()
  form.append('file', fs.createReadStream(file))

  const query = []
  if (install) query.push('install=1')
  if (release) query.push('release=1')
  // const stream = fs.createReadStream(file)
  fetch(`${url}api/sys/pack${ query.length ? `?${query.join('&')}` : ''}`, {
    method: 'POST',
    body: form
  })
    .then(res => res.json())
    .then(json => console.log(json))

}

exports.commit = (release = false) => {
  // commit to git after prepare

  if (release) {
    // tag the commit as well
  }
}


exports.updateDeps = () => {
  // check latest version of deps from registry
}

// fetch('http://localhost:3000/api/system/info', {
//   headers: { 'Authorization': 'Bearer eyJhbGciOiJFUzI1NiIsInR5cCI6IkpXVCJ9.eyJpc3MiOiJxY3Rlc3QuZGV2LmRhdHVtY29ycC5jb20iLCJzdWIiOiJpaHA0SHNuMFRCUzkyV0pGcjZtdTNnIiwibmJmIjoxNTczMTM4Nzg5LCJpYXQiOjE1NzMxMzg3ODksImV4cCI6MTU3NTczMDc4OSwianRpIjoiQlAyQXBCTjAiLCJhdWQiOiJBQUFBQXdBQVFBQ0FBQUFBQUFBQUFRIn0.zvEbWautsdOZFDDF0pnXbbgsw5Ughv2ov2WEKzigwiZWSTv2RCua4OsxNmWB6p0e61LYBo10wHUjZOoRVe4iYw' }
// })

// fetch('https://qctest.dev.datumcorp.com/api/system/info', {
//   headers: { 'Authorization': 'Bearer eyJhbGciOiJFUzI1NiIsInR5cCI6IkpXVCJ9.eyJpc3MiOiJodWIuZGF0dW1jb3JwLmNvbSIsInN1YiI6ImhYVTJMN3d3U0thQ2Nac050dlltTUEiLCJuYmYiOjE1NzMwOTc4MzUsImlhdCI6MTU3MzA5NzgzNSwiZXhwIjoxNTc1Njg5ODM1LCJqdGkiOiJwc3hOWkxwUCIsImF1ZCI6IkRJYXdacUJMUlkyUC1qRWZIQThRNFEifQ.ekHdcOIZ2hXMvy7TaiwzpuIj4nBGIRoY6JLuuU3neMmbenTFfhiV6mNM94csKecZfR_cr-vJhLDu7hPKQ9wszA' }
//   // headers: { 'Authorization': 'Bearer eyJhbGciOiJFUzI1NiIsInR5cCI6IkpXVCJ9.eyJpc3MiOiJxY3Rlc3QuZGV2LmRhdHVtY29ycC5jb20iLCJzdWIiOiJlRzNlT0cwb1IxQzVxRGZEYnU1STBRIiwibmJmIjoxNTczMDk4MTY0LCJpYXQiOjE1NzMwOTgxNjQsImV4cCI6MTU3NTY5MDE2NCwianRpIjoiTWdhU3dOSmEiLCJhdWQiOiJBQUFBQXdBQVFBQ0FBQUFBQUFBQUFRIn0.rRwsyRmQWM8sSEKs2dIcQ-RsJJ4id4T_qeu_vaoaxXYcWb-B5g7T6GwJfE_aQB5yRIF602xUiWvypVXhL97h1Q' }
// })
// .then(res => res.json())
// .then(dat => console.log('Response', dat))
